﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using Datastream.DswsApi;

namespace SampleCode.DswsApi.Net40
{
    /// <summary>
    /// Contains sample code related to miscallaneous
    /// </summary>
    internal class Miscallaneous
    {
        #region Members

        public string Ping()
        {
            try
            {
                DSClient.DataService.Ping();
                return "Ping Succeeded";
            }
            catch (DSException excep)
            {
                return string.Format("Ping Failed.{0}{1}", Environment.NewLine, excep.ToString());
            }

        } //end Ping

        public string DataAccess_Exception()
        {
            try
            {
                // Handle contract errors by catching DSException
                // Here an error request is created (A request which has no instrument)

                // Create the error request 
                var errorRequest = new DSDataRequest()
                {
                    Instrument = new DSInstrument(string.Empty),
                    DataTypes = new DSDataTypes("PL"),
                    Date = new DSSnapshotDate(DSDateType.Absolute(DateTime.Now)),
                };

                // Execute the request
                DSClient.DataService.GetData(errorRequest);

                return "WARNING! THIS CODE SHOULD NOT HAVE BEEN REACHED";

            }
            catch (DSException excep)
            {
                return string.Format("An (expected) error has occured.{0}Message: {1}{0}ErrorCode: {2}", Environment.NewLine, excep.Message, excep.Code);
            }
            

        } //end DataAccess_Exception

        public string DataAccess_DataError()
        {
            // Handle data errors by checking the response
            // Here an invalid datatype is used 

            // Create the error request (a datatype which doesnt exist)
            var errorRequest = new DSDataRequest()
            {
                Instrument = new DSInstrument("VOD"),
                DataTypes = new DSDataTypes("JUNK"),
                Date = new DSTimeSeriesDate(DSDateType.Relative(-30), DSDateType.Relative(-10), DSDateFrequency.Daily),
            };


            // Execute the request
            var response = DSClient.DataService.GetData(errorRequest);

            // Check the response for error
            var responseValue = response["JUNK"]["VOD"];
            if (responseValue.Type == DSSymbolResponseValueType.Error)
                return string.Format("Data Error: {0}", responseValue.Value.ToString());
            else
                return responseValue.Value.ToString();

        } //end DataAccess_DataError
        


        public void API_ShutDown()
        {
            // Optionally call shutdown
            DSClient.ShutDown();

        } //end ShutDown

        

        

        #endregion

    }
}
