﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.Reflection;

namespace SampleCode.DswsApi.Net40
{
    #region Samples Infrastructure Support

    /// <summary>
    /// This class is present for supporting the sample project infrastructure.
    /// To check the actual samples, view the CodeSamples folder!
    /// </summary>
    public partial class MainWindow : Window
    {
        #region Member fields

        #endregion

        #region Constructor

        public MainWindow()
        {
            InitializeComponent();
        }

        #endregion

        #region Methods

        

        private void OnExecuteCompleted(string title, string contents)
        {
            var wnd = new ResultDetailsWindow()
            {
                FieldInfo = title,
                FieldValue = contents,
                Owner = this,
            };
            wnd.ShowDialog();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            var codeItems = SampleCodeItemViewModel.Load();

            // Simple initialization of a two depth tree (TODO: HierarchicalDataTemplate)
            foreach (var grp in codeItems.GroupBy(x => x.Group))
            {
                var grpTVItem = new TreeViewItem() { Header = grp.Key, IsExpanded = true };
                foreach (var item in grp)
                    grpTVItem.Items.Add(new TreeViewItem() { Header = item.Name, DataContext = item });
                this.leftTreeView.Items.Add(grpTVItem);
            }

            ((TreeViewItem)((TreeViewItem)this.leftTreeView.Items[0]).Items[0]).IsSelected = true;
        }

        private void executeButton_Click(object sender, RoutedEventArgs e)
        {
            var tvItem = this.leftTreeView.SelectedItem as TreeViewItem;
            if (tvItem == null || !(tvItem.DataContext is SampleCodeItemViewModelBase))
                return;

            var item = (SampleCodeItemViewModelBase)tvItem.DataContext;
            object context = item is SampleCodeItemViewModel_Arbitrary_GetData ? this.arbitraryGetDataContentControl.Content : item is SampleCodeItemViewModel_Arbitrary_GetDataBundle ? this.arbitraryGetDataBundleItemsControl.ItemsSource : null; 

            Action<string,string> executeCompleted = (title, contents) =>
            {
                this.Dispatcher.Invoke(new Action<string,string>(OnExecuteCompleted), new[] { title, contents });
            };
            item.ExecuteCode(executeCompleted, context);
        }

        

        private void leftTreeView_SelectedItemChanged(object sender, RoutedPropertyChangedEventArgs<object> e)
        {
            var tvItem = e.NewValue as TreeViewItem;
            if (tvItem == null || !(tvItem.DataContext is SampleCodeItemViewModelBase))
                return;

            var item = (SampleCodeItemViewModelBase)tvItem.DataContext;
            this.titleLabel.Content = item.Title;
            this.descriptionTextBlock.Text = item.Description;

            foreach (FrameworkElement child in this.dataPanel.Children)
                child.Visibility = Visibility.Collapsed;

            if (item is SampleCodeItemViewModel)
            {
                this.sampleCodePanel.Visibility = System.Windows.Visibility.Visible;
                this.samplecodeFileLabel.Text = ((SampleCodeItemViewModel)item).SampleCodeFile;
                this.samplecodeMethodLabel.Text = ((SampleCodeItemViewModel)item).ExecuteMethod;
                this.samplecodeTextBox.Text = ((SampleCodeItemViewModel)item).GetSampleCode();
                
            }
            else if (item is SampleCodeItemViewModel_Arbitrary_GetData)
            {
                this.arbitraryGetDataPanel.Visibility = System.Windows.Visibility.Visible;
                this.arbitraryGetDataContentControl.Content = new ArbitraryDataItem();
            }
            else if (item is SampleCodeItemViewModel_Arbitrary_GetDataBundle)
            {
                this.arbitraryGetDataBundlePanel.Visibility = System.Windows.Visibility.Visible;
                this.arbitraryGetDataBundleItemsControl.ItemsSource = new ObservableCollection<ArbitraryDataItem>(new[] { new ArbitraryDataItem() });
            }

        }

        private void getdatabundle_AddButton_Click(object sender, RoutedEventArgs e)
        {
            ((ObservableCollection<ArbitraryDataItem>)this.arbitraryGetDataBundleItemsControl.ItemsSource).Add(new ArbitraryDataItem());
        }

        private void getdatabundle_ClearButton_Click(object sender, RoutedEventArgs e)
        {
            this.arbitraryGetDataBundleItemsControl.ItemsSource = new ObservableCollection<ArbitraryDataItem>(new[] { new ArbitraryDataItem() });
        }

        #endregion
    }

    #endregion
}
