﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using System.IO;
using System.ComponentModel;
using System.Diagnostics;
using Datastream.DswsApi;

namespace SampleCode.DswsApi.Net40
{
    #region Samples Infrastructure Support

    /// <summary>
    /// This class is present for supporting the sample project infrastructure.
    /// To check the actual samples, view the CodeSamples folder!
    /// </summary>
    internal abstract class SampleCodeItemViewModelBase
    {
        #region Members

        public string Group
        {
            get;
            set;
        }

        public string Name
        {
            get;
            set;
        }

        public string Title
        {
            get;
            set;
        }

        public string Description
        {
            get;
            set;
        }

        public string ExecuteType
        {
            get;
            set;
        }

        public string ExecuteMethod
        {
            get;
            set;
        }

      
        


        public static IEnumerable<SampleCodeItemViewModelBase> Load()
        {
            return new SampleCodeItemViewModelBase[]
            {
                new SampleCodeItemViewModel()
                {
                    Group = "Initialize",
                    Name = "APIOptions (Config)",
                    Title = "Init API Options From Config",
                    Description = "Demonstrates initializing API options through config file. You can also configure options through code. You will configure your options in the appSettings element of your config file (Web.config/app.config). Make sure you edit the username and password settings in file app.config before clicking Execute.",
                    ExecuteType = "SampleCode.DswsApi.Net40.Initialize",
                    ExecuteMethod = "APIOptions_Config",
                    SampleCodeFile = "Initialize.cs",
                    SampleCodeBeginEndMarker = new[] { new KeyValuePair<string,string>("public void APIOptions_Config()", "//end APIOptions_Config") },
                },
                new SampleCodeItemViewModel()
                {
                    Group = "Initialize",
                    Name = "APIOptions (Code)",
                    Title = "Init API Options From Code",
                    Description = "Demonstrates initializing API options through code. You can also configure options through config file. You will perform this operation typically only once when your application starts. Make sure you edit the username and password in code and re-compile before clicking Execute.",
                    ExecuteType = "SampleCode.DswsApi.Net40.Initialize",
                    ExecuteMethod = "APIOptions_Code",
                    SampleCodeFile = "Initialize.cs",
                    SampleCodeBeginEndMarker = new[] { new KeyValuePair<string,string>("public void APIOptions_Code()", "//end APIOptions_Code") },
                },
                new SampleCodeItemViewModel()
                {
                    Group = "Initialize",
                    Name = "Tracing (Config)",
                    Title = "Init API Tracing From Config",
                    Description = "Demonstrates initializing API tracing through config file. You can also configure tracing through code. You will configure trace level (such as Error, Information) and the destination listener (such as FlatFile) in the diagnostics element of your config file (Web.config/app.config).",
                    ExecuteType = "SampleCode.DswsApi.Net40.Initialize",
                    ExecuteMethod = "APIOptions_Config",
                    SampleCodeFile = "Initialize.cs",
                    SampleCodeBeginEndMarker = new[] { new KeyValuePair<string,string>("public void API_Tracing_Config()", "//end API_Tracing_Config") },
                },
                new SampleCodeItemViewModel()
                {
                    Group = "Initialize",
                    Name = "Tracing (Code)",
                    Title = "Init API Tracing From Code",
                    Description = "Demonstrates initializing API tracing through code. You can also configure tracing through config file. You will configure trace level (such as Error, Information) and the destination listener (such as FlatFile).",
                    ExecuteType = "SampleCode.DswsApi.Net40.Initialize",
                    ExecuteMethod = "API_Tracing_Code",
                    SampleCodeFile = "Initialize.cs",
                    SampleCodeBeginEndMarker = new[] { new KeyValuePair<string,string>("public void API_Tracing_Code()", "//end API_Tracing_Code") },
                },
                new SampleCodeItemViewModel()
                {
                    Group = "Initialize",
                    Name = "Explicit Logon",
                    Title = "Logon Explicitly",
                    Description = "Demonstrates logging on explicitly. You need not usually call this method since whenever a data access method is called and a valid token is not available, a logon will happen implicitly. However you can use this method in case you need to check if your credentials are valid upfront. This method will throw an exception if your credentials are invalid.",
                    ExecuteType = "SampleCode.DswsApi.Net40.Initialize",
                    ExecuteMethod = "Logon",
                    SampleCodeFile = "Initialize.cs",
                    SampleCodeBeginEndMarker = new[] { new KeyValuePair<string,string>("public string Logon()", "//end Logon") },
                },
                new SampleCodeItemViewModel()
                {
                    Group = "GetData",
                    Name = "Snapshot (Single)",
                    Title = "GetData - Snapshot (Single)",
                    Description = "Demonstrates getting data for a snapshot request. This example is for a single instrument using relative dates. You can access the response using indexers or enumerate the responses. You can alter the parameters in code and re-compile before clicking Execute.",
                    ExecuteType = "SampleCode.DswsApi.Net40.GetData",
                    ExecuteMethod = "GetData_SnapshotSingle",
                    SampleCodeFile = "GetData.cs",
                    SampleCodeBeginEndMarker = new[] { new KeyValuePair<string,string>("public string GetData_SnapshotSingle()", "//end GetData_SnapshotSingle") },
                },
                new SampleCodeItemViewModel()
                {
                    Group = "GetData",
                    Name = "Snapshot (Multiple)",
                    Title = "GetData - Snapshot (Multiple)",
                    Description = "Demonstrates getting data for a snapshot request. This example is for multiple instruments using absolute dates and additionally returning datatype expanded name. You can access the response using indexers or enumerate the responses. You can alter the parameters in code and re-compile before clicking Execute.",
                    ExecuteType = "SampleCode.DswsApi.Net40.GetData",
                    ExecuteMethod = "GetData_SnapshotMulti",
                    SampleCodeFile = "GetData.cs",
                    SampleCodeBeginEndMarker = new[] { new KeyValuePair<string,string>("public string GetData_SnapshotMulti()", "//end GetData_SnapshotMulti") },
                },
                new SampleCodeItemViewModel()
                {
                    Group = "GetData",
                    Name = "Snapshot (Expression)",
                    Title = "GetData - Snapshot (Expression)",
                    Description = "Demonstrates getting data for a snapshot request. This example is for an expression using literal dates. You can access the response using indexers or enumerate the responses. You can alter the parameters in code and re-compile before clicking Execute.",
                    ExecuteType = "SampleCode.DswsApi.Net40.GetData",
                    ExecuteMethod = "GetData_SnapshotExpression",
                    SampleCodeFile = "GetData.cs",
                    SampleCodeBeginEndMarker = new[] { new KeyValuePair<string,string>("public string GetData_SnapshotExpression()", "//end GetData_SnapshotExpression"), new KeyValuePair<string,string>("private string GetResponseString(DSDataResponse response)", "//end GetResponseString") },
                },
                new SampleCodeItemViewModel()
                {
                    Group = "GetData",
                    Name = "Snapshot (List)",
                    Title = "GetData - Snapshot (List)",
                    Description = "Demonstrates getting data for a snapshot request. This example is for list instrument and not specifying dates. You can access the response using indexers or enumerate the responses. You can alter the parameters in code and re-compile before clicking Execute.",
                    ExecuteType = "SampleCode.DswsApi.Net40.GetData",
                    ExecuteMethod = "GetData_SnapshotList",
                    SampleCodeFile = "GetData.cs",
                    SampleCodeBeginEndMarker = new[] { new KeyValuePair<string,string>("public string GetData_SnapshotList()", "//end GetData_SnapshotList"), new KeyValuePair<string,string>("private string GetResponseString(DSDataResponse response)", "//end GetResponseString") },
                },
                new SampleCodeItemViewModel()
                {
                    Group = "GetData",
                    Name = "TimeSeries (Single)",
                    Title = "GetData - TimeSeries (Single)",
                    Description = "Demonstrates getting data for a TimeSeries request. This example is for a single instrument using relative dates. You can access the response using indexers or enumerate the responses. You can alter the parameters in code and re-compile before clicking Execute.",
                    ExecuteType = "SampleCode.DswsApi.Net40.GetData",
                    ExecuteMethod = "GetData_TimeSeriesSingle",
                    SampleCodeFile = "GetData.cs",
                    SampleCodeBeginEndMarker = new[] { new KeyValuePair<string,string>("public string GetData_TimeSeriesSingle()", "//end GetData_TimeSeriesSingle") },
                },
                new SampleCodeItemViewModel()
                {
                    Group = "GetData",
                    Name = "TimeSeries (Multiple)",
                    Title = "GetData - TimeSeries (Multiple)",
                    Description = "Demonstrates getting data for a TimeSeries request. This example is for multiple instruments using absolute dates and additionally returning datatype expanded name. You can access the response using indexers or enumerate the responses. You can alter the parameters in code and re-compile before clicking Execute.",
                    ExecuteType = "SampleCode.DswsApi.Net40.GetData",
                    ExecuteMethod = "GetData_TimeSeriesMulti",
                    SampleCodeFile = "GetData.cs",
                    SampleCodeBeginEndMarker = new[] { new KeyValuePair<string,string>("public string GetData_TimeSeriesMulti()", "//end GetData_TimeSeriesMulti"), new KeyValuePair<string,string>("private string GetResponseString(DSDataResponse response)", "//end GetResponseString") },
                },
                new SampleCodeItemViewModel()
                {
                    Group = "GetData",
                    Name = "TimeSeries (Expression)",
                    Title = "GetData - TimeSeries (Expression)",
                    Description = "Demonstrates getting data for a TimeSeries request. This example is for an expression using literal dates. You can access the response using indexers or enumerate the responses. You can alter the parameters in code and re-compile before clicking Execute.",
                    ExecuteType = "SampleCode.DswsApi.Net40.GetData",
                    ExecuteMethod = "GetData_TimeSeriesExpression",
                    SampleCodeFile = "GetData.cs",
                    SampleCodeBeginEndMarker = new[] { new KeyValuePair<string,string>("public string GetData_TimeSeriesExpression()", "//end GetData_TimeSeriesExpression"), new KeyValuePair<string,string>("private string GetResponseString(DSDataResponse response)", "//end GetResponseString") },
                },
                new SampleCodeItemViewModel_Arbitrary_GetData()
                {
                    Group = "GetData",
                    Name = "Arbitrary",
                    Title = "GetData - Arbitrary",
                    Description = "Demonstrates getting data for an arbitrary user-entered request. This option is present to enter free parameters for instrument, datatypes, date etc. It is used as a testing tool rather than to illustrate sample code.",
                    ExecuteType = "SampleCode.DswsApi.Net40.GetData",
                    ExecuteMethod = "GetData_Arbitrary",
                },
                new SampleCodeItemViewModel()
                {
                    Group = "GetDataBundle",
                    Name = "Snapshot (All)",
                    Title = "GetDataBundle - All Snapshot",
                    Description = "Demonstrates getting data for multiple snapshot requests. This example uses different snapshot requests. You can alter the parameters in code and re-compile before clicking Execute.",
                    ExecuteType = "SampleCode.DswsApi.Net40.GetDataBundle",
                    ExecuteMethod = "GetDataBundle_Snapshot",
                    SampleCodeFile = "GetDataBundle.cs",
                    SampleCodeBeginEndMarker = new[] { new KeyValuePair<string,string>("public string GetDataBundle_Snapshot()", "//end GetDataBundle_Snapshot"), new KeyValuePair<string,string>("private string GetResponsesString(IEnumerable<DSDataResponse> responses)", "//end GetResponsesString"), new KeyValuePair<string,string>("private string GetResponseString(DSDataResponse response)", "//end GetResponseString") },
                },
                new SampleCodeItemViewModel()
                {
                    Group = "GetDataBundle",
                    Name = "TimeSeries (All)",
                    Title = "GetDataBundle - All TimeSeries",
                    Description = "Demonstrates getting data for multiple timeseries requests. This example uses different timeseries requests. You can alter the parameters in code and re-compile before clicking Execute.",
                    ExecuteType = "SampleCode.DswsApi.Net40.GetDataBundle",
                    ExecuteMethod = "GetDataBundle_TimeSeries",
                    SampleCodeFile = "GetDataBundle.cs",
                    SampleCodeBeginEndMarker = new[] { new KeyValuePair<string,string>("public string GetDataBundle_TimeSeries()", "//end GetDataBundle_TimeSeries"), new KeyValuePair<string,string>("private string GetResponsesString(IEnumerable<DSDataResponse> responses)", "//end GetResponsesString"), new KeyValuePair<string,string>("private string GetResponseString(DSDataResponse response)", "//end GetResponseString") },
                },
                new SampleCodeItemViewModel()
                {
                    Group = "GetDataBundle",
                    Name = "Snapshot & TimeSeries",
                    Title = "GetDataBundle - Snapshot & TimeSeries Mix",
                    Description = "Demonstrates getting data for multiple snapshot/timeseries requests. This example uses different snapshot & timeseries requests. You can alter the parameters in code and re-compile before clicking Execute.",
                    ExecuteType = "SampleCode.DswsApi.Net40.GetDataBundle",
                    ExecuteMethod = "GetDataBundle_Mixed",
                    SampleCodeFile = "GetDataBundle.cs",
                    SampleCodeBeginEndMarker = new[] { new KeyValuePair<string,string>("public string GetDataBundle_Mixed()", "//end GetDataBundle_Mixed"), new KeyValuePair<string,string>("private string GetResponsesString(IEnumerable<DSDataResponse> responses)", "//end GetResponsesString"), new KeyValuePair<string,string>("private string GetResponseString(DSDataResponse response)", "//end GetResponseString") },
                },
                new SampleCodeItemViewModel_Arbitrary_GetDataBundle()
                {
                    Group = "GetDataBundle",
                    Name = "Arbitrary",
                    Title = "GetDataBundle - Arbitrary",
                    Description = "Demonstrates getting data bundle for an arbitrary user-entered request. This option is present to enter free parameters for instrument, datatypes, date etc. It is used as a testing tool rather than to illustrate sample code.",
                    ExecuteType = "SampleCode.DswsApi.Net40.GetDataBundle",
                    ExecuteMethod = "GetDataBundle_Arbitrary",
                },
                new SampleCodeItemViewModel()
                {
                    Group = "Asynchronous",
                    Name = "GetData",
                    Title = "Asynchronous - GetData",
                    Description = "Demonstrates getting data for a request asynchronously. This example uses the .NET 4 task based asynchronous pattern. You can alter the parameters in code and re-compile before clicking Execute.",
                    ExecuteType = "SampleCode.DswsApi.Net40.AsyncAccess",
                    ExecuteMethod = "GetDataAsync",
                    SampleCodeFile = "AsyncAccess.cs",
                    SampleCodeBeginEndMarker = new[] { new KeyValuePair<string,string>("public void GetDataAsync()", "//end GetDataAsync"), new KeyValuePair<string,string>("private static string GetResponseString(DSDataResponse response)", "//end GetResponseString") },
                },
                new SampleCodeItemViewModel()
                {
                    Group = "Asynchronous",
                    Name = "GetDataBundle",
                    Title = "Asynchronous - GetDataBundle",
                    Description = "Demonstrates getting data for multiple requests asynchronously. This example uses the .NET 4 task based asynchronous pattern. You can alter the parameters in code and re-compile before clicking Execute.",
                    ExecuteType = "SampleCode.DswsApi.Net40.AsyncAccess",
                    ExecuteMethod = "GetDataBundleAsync",
                    SampleCodeFile = "AsyncAccess.cs",
                    SampleCodeBeginEndMarker = new[] { new KeyValuePair<string,string>("public void GetDataBundleAsync()", "//end GetDataBundleAsync"), new KeyValuePair<string,string>("private static string GetResponseString(DSDataResponse response)", "//end GetResponseString") },
                },
                 new SampleCodeItemViewModel_Arbitrary_GetData()
                {
                    Group = "Asynchronous",
                    Name = "GetData (Arbitrary)",
                    Title = "Asynchronous - Arbitrary GetData",
                    Description = "Demonstrates getting data for an arbitrary user-entered request. This option is present to enter free parameters for instrument, datatypes, date etc. It is used as a testing tool rather than to illustrate sample code.",
                    ExecuteType = "SampleCode.DswsApi.Net40.AsyncAccess",
                    ExecuteMethod = "GetDataAsync_Arbitrary",
                },
                new SampleCodeItemViewModel_Arbitrary_GetDataBundle()
                {
                    Group = "Asynchronous",
                    Name = "GetDataBundle (Arbitrary)",
                    Title = "Asynchronous - Arbitrary GetDataBundle",
                    Description = "Demonstrates getting data bundle for an arbitrary user-entered request asynchronously. This option is present to enter free parameters for instrument, datatypes, date etc. It is used as a testing tool rather than to illustrate sample code.",
                    ExecuteType = "SampleCode.DswsApi.Net40.AsyncAccess",
                    ExecuteMethod = "GetDataBundleAsync_Arbitrary",
                },
                new SampleCodeItemViewModel()
                {
                    Group = "Miscallaneous",
                    Name = "Exceptions",
                    Title = "Handle Exceptions",
                    Description = "Demonstrates handling exceptions. Exceptions occur for conditions such as connection failure, contract violations, invalid arguments etc. Note that this is different from data related errors such as non-availability of data for the requested parameters. You will typically catch a DSException to handle the errors.",
                    ExecuteType = "SampleCode.DswsApi.Net40.Miscallaneous",
                    ExecuteMethod = "DataAccess_Exception",
                    SampleCodeFile = "Miscallaneous.cs",
                    SampleCodeBeginEndMarker = new[] { new KeyValuePair<string,string>("public string DataAccess_Exception()", "//end DataAccess_Exception") },
                },
                new SampleCodeItemViewModel()
                {
                    Group = "Miscallaneous",
                    Name = "Data Error",
                    Title = "Handle Data Errors",
                    Description = "Demonstrates handling data errors. Data related errors could occur for multiple reasons including unavailability of data, a list that does not exist etc. Note that this is different from exceptions that could occur for reasons such as connection failure, contract violations, invalid arguments etc. You will typically examine the DSSymbolResponseValue's Type to check if it is an error.",
                    ExecuteType = "SampleCode.DswsApi.Net40.Miscallaneous",
                    ExecuteMethod = "DataAccess_DataError",
                    SampleCodeFile = "Miscallaneous.cs",
                    SampleCodeBeginEndMarker = new[] { new KeyValuePair<string,string>("public string DataAccess_DataError()", "//end DataAccess_DataError") },
                },
                new SampleCodeItemViewModel()
                {
                    Group = "Miscallaneous",
                    Name = "Ping",
                    Title = "Ping for Connectivity Check",
                    Description = "Demonstrates pinging for connectivity check. You can use this method to monitor connectivity to the server. This functionality is usually not needed though.",
                    ExecuteType = "SampleCode.DswsApi.Net40.Miscallaneous",
                    ExecuteMethod = "Ping",
                    SampleCodeFile = "Miscallaneous.cs",
                    SampleCodeBeginEndMarker = new[] { new KeyValuePair<string,string>("public string Ping()", "//end Ping") },
                },
                new SampleCodeItemViewModel()
                {
                    Group = "Miscallaneous",
                    Name = "API ShutDown",
                    Title = "ShutDown API",
                    Description = "Demonstrates shutting down API. You will perform this operation typically only once when your application shutsdown. This allows the API to clean-up internal state (if needed). Calling this method is optional though.",
                    ExecuteType = "SampleCode.DswsApi.Net40.Miscallaneous",
                    ExecuteMethod = "API_ShutDown",
                    SampleCodeFile = "Miscallaneous.cs",
                    SampleCodeBeginEndMarker = new[] { new KeyValuePair<string,string>("public void API_ShutDown()", "//end ShutDown") },
                },

                
            };
        }


        public void ExecuteCode(Action<string, string> onExecuteCompleted, object context)
        {
            if (string.IsNullOrEmpty(this.ExecuteType) || string.IsNullOrEmpty(this.ExecuteMethod))
                onExecuteCompleted(this.Title, "There was no sample code to execute.");

            try
            {
                var type = Type.GetType(this.ExecuteType);
                if (type == null)
                    throw new Exception(string.Format("Unable to get type '{0}'", this.ExecuteType));

                var instance = type.GetConstructor(Type.EmptyTypes).Invoke(null);

                var method = type.GetMethod(this.ExecuteMethod);
                if (method == null)
                    throw new Exception(string.Format("Unable to get method '{0}'", this.ExecuteMethod));


                if (instance is IAsyncAccess)
                {
                    DateTime datetimeStarted = DateTime.Now;
                    ((IAsyncAccess)instance).AsyncOperationCompleted += (excep, res) =>
                    {
                        onExecuteCompleted(string.Format("{0} (Started @{1})", this.Title, datetimeStarted.ToString("HH:mm:ss")), excep != null ? string.Format("There was an error while executing the sample code. {0} Error: {0} {1}", System.Environment.NewLine, excep.ToString()) : res.ToString());
                    };
                }

                var result = method.Invoke(instance, GetExecuteArgs(context, instance, method));
                if (result != null)
                    onExecuteCompleted(this.Title, result.ToString());
            }
            catch (Exception excep)
            {
                var excepToUse = excep;
                if (excep is System.Reflection.TargetInvocationException)
                    excepToUse = ((System.Reflection.TargetInvocationException)excep).InnerException;
                onExecuteCompleted(this.Title, string.Format("There was an error while executing the sample code. {0} Error: {0} {1}", System.Environment.NewLine, excepToUse.ToString()));
            }
        }


        protected abstract object[] GetExecuteArgs(object context, object instance, System.Reflection.MethodInfo method);

        #endregion

    }

    /// <summary>
    /// This class is present for supporting the sample project infrastructure.
    /// To check the actual samples, view the CodeSamples folder!
    /// </summary>
    internal class SampleCodeItemViewModel : SampleCodeItemViewModelBase
    {
        #region Members

        public string SampleCodeFile
        {
            get;
            set;
        }

        public KeyValuePair<string, string>[] SampleCodeBeginEndMarker
        {
            get;
            set;
        }

        public string GetSampleCode()
        {
            if (string.IsNullOrEmpty(this.SampleCodeFile) || this.SampleCodeBeginEndMarker == null)
                return "There was no sample code configured.";

            try
            {
                string fileName = this.SampleCodeFile;
                if (!File.Exists(this.SampleCodeFile))
                {
                    fileName = Path.Combine(@"..\..\CodeSamples", this.SampleCodeFile);
                    if (!File.Exists(fileName))
                        throw new Exception(string.Format("The file '{0}' was not present.", fileName));
                }

                using (var file = new StreamReader(fileName))
                {
                    string contents = file.ReadToEnd();     // TODO: Currently assuming the file will be small and not reading line by line
                    var sbCode = new StringBuilder();
                    for (int idx = 0; idx < this.SampleCodeBeginEndMarker.Length; idx++)
                    {
                        var kv = this.SampleCodeBeginEndMarker[idx];
                        int beginIndex = contents.IndexOf(kv.Key);
                        if (beginIndex == -1)
                            throw new Exception(string.Format("The begin index marker '{0}' was not present.", kv.Key));

                        int endIndex = contents.IndexOf(kv.Value);
                        if (endIndex == -1)
                            throw new Exception(string.Format("The end index marker '{0}' was not present.", kv.Value));

                        if (endIndex <= beginIndex)
                            throw new Exception(string.Format("The end index marker '{0}' was lesser than start index marker '{1}.", kv.Value, kv.Key));

                        var code = contents.Substring(beginIndex, (endIndex - beginIndex /*+ kv.Value.Length*/));

                        // Nasty hack...
                        code = code.PadLeft(code.Length + 8, ' ');
                        sbCode.Append(code);

                        if (idx != this.SampleCodeBeginEndMarker.Length - 1)
                        {
                            for (int line = 0; line < 2; line++)
                                sbCode.AppendLine();
                        }
                    }

                    return sbCode.ToString();
                }

            }
            catch (Exception excep)
            {
                return string.Format("There was an error while loading the sample code. Make sure the sample code project is present and run as distributed. {0} Error: {0} {1}", System.Environment.NewLine, excep.ToString());
            }


        }

        protected override object[] GetExecuteArgs(object context, object instance, System.Reflection.MethodInfo method)
        {
            return null;
        }

        #endregion
    }

    /// <summary>
    /// This class is present for supporting the sample project infrastructure.
    /// To check the actual samples, view the CodeSamples folder!
    /// </summary>
    internal class SampleCodeItemViewModel_Arbitrary_GetData : SampleCodeItemViewModelBase
    {
        #region Members

        protected override object[] GetExecuteArgs(object context, object instance, System.Reflection.MethodInfo method)
        {
            Debug.Assert(context is ArbitraryDataItem);
            return new[] {  ((ArbitraryDataItem)context).ToDSDataRequest() };
        }

        #endregion
    }

    /// <summary>
    /// This class is present for supporting the sample project infrastructure.
    /// To check the actual samples, view the CodeSamples folder!
    /// </summary>
    internal class SampleCodeItemViewModel_Arbitrary_GetDataBundle : SampleCodeItemViewModelBase
    {
        #region Members

        protected override object[] GetExecuteArgs(object context, object instance, System.Reflection.MethodInfo method)
        {
            Debug.Assert(context is IEnumerable<ArbitraryDataItem>);
            return new[] { ((IEnumerable<ArbitraryDataItem>)context).Select(x => x.ToDSDataRequest()).ToArray() };
        }

        #endregion
    }

    /// <summary>
    /// This class is present for supporting the sample project infrastructure.
    /// To check the actual samples, view the CodeSamples folder!
    /// </summary>
    internal class ArbitraryDataItem : INotifyPropertyChanged
    {
        #region Members

        private static readonly string[] SEPERATORS = new[] { ",", ";", "|" };

        

        public ArbitraryDataItem()
        {
            this.InstrumentSepIndex = 0;
            this.DataTypeSepIndex = 0;
            this.DateTypeIndex = 0;
        }

        
        

        public string Instrument
        {
            get;
            set;
        }

        public int InstrumentSepIndex
        {
            get;
            set;
        }

        public bool IsExpression
        {
            get;
            set;
        }

        public bool IsList
        {
            get;
            set;
        }

        public string DataType
        {
            get;
            set;
        }

        public int DataTypeSepIndex
        {
            get;
            set;
        }

        public string StartDate
        {
            get;
            set;
        }

        public string EndDate
        {
            get;
            set;
        }

        public string Frequency
        {
            get;
            set;
        }

        public int DateTypeIndex
        {
            get;
            set;
        }

        public bool Response_DataTypeName
        {
            get;
            set;
        }

        public bool Response_InstrumentName
        {
            get;
            set;
        }

        public bool Response_Currency
        {
            get;
            set;
        }

        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged(string propName)
        {
            if (this.PropertyChanged != null)
                this.PropertyChanged(this, new PropertyChangedEventArgs(propName));
        }

        public DSDataRequest ToDSDataRequest()
        {
            DSInstrument dsInstr = null;
            if (this.IsExpression)
                dsInstr = new DSInstrument(this.Instrument, DSInstrumentProperties.IsExpression);
            else if (this.Instrument != null && this.Instrument.Contains(SEPERATORS[this.InstrumentSepIndex]))
                dsInstr = new DSInstrument(this.Instrument.Split(SEPERATORS[this.InstrumentSepIndex].ToCharArray(), StringSplitOptions.RemoveEmptyEntries));
            else if (this.IsList)
                dsInstr = new DSInstrument(this.Instrument, DSInstrumentProperties.IsList);
            else
                dsInstr = new DSInstrument(this.Instrument);


            DSDataTypes dsDataTypes = this.DataType != null ? new DSDataTypes(this.DataType.Split(SEPERATORS[this.DataTypeSepIndex].ToCharArray(), StringSplitOptions.RemoveEmptyEntries)) : null;
            DSDateBase dsDate = this.DateTypeIndex == 0 ? (DSDateBase)new DSSnapshotDate(DSDateType.Literal(this.StartDate))
                : new DSTimeSeriesDate(DSDateType.Literal(this.StartDate), DSDateType.Literal(this.EndDate), this.Frequency == "D" ? DSDateFrequency.Daily : this.Frequency == "M" ? DSDateFrequency.Monthly : this.Frequency == "W" ? DSDateFrequency.Weekly : this.Frequency == "Q" ? DSDateFrequency.Quarterly : this.Frequency == "Y" ? DSDateFrequency.Yearly : DSDateFrequency.None);

            var props = DSDataRequestPropertyTypes.None;
            if (this.Response_DataTypeName)
                props = props | DSDataRequestPropertyTypes.ReturnDataTypeExpandedName;
            if (this.Response_InstrumentName)
                props = props | DSDataRequestPropertyTypes.ReturnInstrumentExpandedName;
            if (this.Response_Currency)
                props = props | DSDataRequestPropertyTypes.ReturnCurrency;

            return new DSDataRequest()
            {
                Instrument = dsInstr,
                DataTypes = dsDataTypes,
                Date = dsDate,
                Properties = new DSDataRequestProperties(props),
                Tag = null,
            };
        }

        #endregion
    }

    /// <summary>
    /// This class is present for supporting the sample project infrastructure.
    /// To check the actual samples, view the CodeSamples folder!
    /// </summary>
    internal interface IAsyncAccess
    {
        #region Members

        event Action<Exception, string> AsyncOperationCompleted;

        #endregion
    }

    #endregion
    
}
